@@ -182,7 +182,8 @@ impl ChildComponent {
         match (&mut element, children.is_empty()) {
             (ftd::Element::Column(_), _)
             | (ftd::Element::Row(_), _)
-            | (ftd::Element::Scene(_), _) => {
+            | (ftd::Element::Scene(_), _)
+            | (ftd::Element::Grid(_), _) => {
                 let instructions = children
                     .iter()
                     .map(|child| {
@@ -220,7 +221,7 @@ impl ChildComponent {
                         .unwrap();
                 }
                 match root.full_name.as_str() {
-                    "ftd#row" | "ftd#column" | "ftd#scene" => {}
+                    "ftd#row" | "ftd#column" | "ftd#scene" | "ftd#grid" => {}
                     t => {
                         return ftd::e2(
                             format!("{} cant have children", t),
@@ -555,6 +556,7 @@ impl ChildComponent {
                         doc,
                         arguments,
                         None,
+                        None,
                     ) {
                         properties.insert(
                             "value".to_string(),
@@ -682,6 +684,20 @@ fn get_conditional_attributes(
         ],
     );
 
+    dictionary.insert("slots".to_string(), vec!["grid-template-areas".to_string()]);
+    dictionary.insert(
+        "slot-widths".to_string(),
+        vec!["grid-template-columns".to_string()],
+    );
+    dictionary.insert(
+        "slot-heights".to_string(),
+        vec!["grid-template-rows".to_string()],
+    );
+    if properties.contains_key("slots") {
+        dictionary.insert("spacing".to_string(), vec!["grid-gap".to_string()]);
+    }
+    dictionary.insert("slot".to_string(), vec!["grid-area".to_string()]);
+
     for (name, value) in properties {
         if !value.conditions.is_empty() {
             let styles = if let Some(styles) = dictionary.get(name) {
@@ -762,6 +778,7 @@ fn get_conditional_attributes(
             "shadow-blur",
             "font-size",
             "border-width",
+            "grid-gap",
         ];
 
         let style_length = vec![
@@ -786,7 +803,15 @@ fn get_conditional_attributes(
             "border-bottom-right-radius",
         ];
 
-        let style_string = vec!["cursor", "position", "align", "background-image"];
+        let style_string = vec![
+            "cursor",
+            "position",
+            "align",
+            "background-image",
+            "grid-template-columns",
+            "grid-template-rows",
+            "grid-area",
+        ];
 
         let style_overflow = vec!["overflow-x", "overflow-y"];
 
@@ -914,6 +939,27 @@ fn get_conditional_attributes(
                 },
                 v => return ftd::e2(format!("expected int, found: {:?}", v), doc_id, line_number),
             }
+        } else if name.eq("grid-template-areas") {
+            match value {
+                ftd::Value::String { text: v, .. } => {
+                    let areas = v.split('|').map(|v| v.trim()).collect::<Vec<&str>>();
+                    let mut css_areas = "".to_string();
+                    for area in areas {
+                        css_areas = format!("{}'{}'", css_areas, area);
+                    }
+                    ftd::ConditionalValue {
+                        value: css_areas,
+                        important: false,
+                    }
+                }
+                v => {
+                    return ftd::e2(
+                        format!("expected string, found: {:?}", v),
+                        doc_id,
+                        line_number,
+                    )
+                }
+            }
         } else {
             return ftd::e2(
                 format!("unknown style name: `{}` value:`{:?}`", name, value),
@@ -1249,6 +1295,9 @@ impl Component {
                 "ftd#scene" => ftd::Element::Scene(ftd::p2::element::scene_from_properties(
                     arguments, doc, condition, is_child, events, all_locals, root_name,
                 )?),
+                "ftd#grid" => ftd::Element::Grid(ftd::p2::element::grid_from_properties(
+                    arguments, doc, condition, is_child, events, all_locals, root_name,
+                )?),
                 _ => unreachable!(),
             };
             Ok(ElementWithContainer {
@@ -1395,6 +1444,9 @@ impl Component {
                 })
                 | ftd::Element::Scene(ftd::Scene {
                     ref mut container, ..
+                })
+                | ftd::Element::Grid(ftd::Grid {
+                    ref mut container, ..
                 }) => {
                     let ElementWithContainer {
                         children,
@@ -1525,9 +1577,10 @@ pub fn recursive_child_component(
         doc,
         arguments,
         None,
+        None,
     )?;
 
-    let recursive_kind = if let ftd::p2::Kind::List { kind } = recursive_property_value.kind() {
+    let recursive_kind = if let ftd::p2::Kind::List { kind, .. } = recursive_property_value.kind() {
         kind.as_ref().to_owned()
     } else {
         return ftd::e2(
@@ -1690,6 +1743,7 @@ pub fn recursive_child_component(
             doc,
             &arguments,
             None,
+            None,
         )?;
         Ok(ftd::component::Property {
             default: Some(property),
@@ -1721,6 +1775,7 @@ fn update_properties(
 ) {
     let default_property = vec![
         "id", "top", "bottom", "left", "right", "align", "scale", "rotate", "scale-x", "scale-y",
+        "slot",
     ];
     for p in default_property {
         if !properties.contains_key(p) {
@@ -1755,7 +1810,8 @@ fn is_component(name: &str) -> bool {
         || (name == "ftd.decimal")
         || (name == "ftd.boolean")
         || (name == "ftd.input")
-        || (name == "ftd.scene"))
+        || (name == "ftd.scene")
+        || (name == "ftd.grid"))
 }
 
 fn assert_no_extra_properties(
@@ -1787,7 +1843,7 @@ fn assert_no_extra_properties(
             || (is_component(name)
                 && vec![
                     "id", "top", "bottom", "left", "right", "align", "scale", "rotate", "scale-x",
-                    "scale-y",
+                    "scale-y", "slot",
                 ]
                 .contains(&key)))
         {
@@ -1911,6 +1967,7 @@ pub fn read_properties(
                 doc,
                 arguments,
                 Some(source.clone()),
+                None,
             )?;
             let nested_properties = if let ftd::PropertyValue::Reference {
                 kind: ftd::p2::Kind::UI { .. },
@@ -2016,6 +2073,7 @@ pub fn read_properties(
             "scale-y".to_string(),
             ftd::p2::Kind::decimal().into_optional(),
         );
+        default_argument.insert("slot".to_string(), ftd::p2::Kind::string().into_optional());
 
         for (key, arg) in default_argument {
             root_arguments.entry(key).or_insert(arg);
@@ -2038,6 +2096,7 @@ fn root_properties_from_inherits(
             doc,
             arguments,
             None,
+            None,
         )?;
         root_properties.insert(
             inherit,
