@@ -251,6 +251,7 @@ impl ftd::Element {
         match self {
             Self::Row(i) => (i.to_node(doc_id)),
             Self::Scene(i) => (i.to_node(doc_id)),
+            Self::Grid(i) => (i.to_node(doc_id)),
             Self::Text(i) => (i.to_node(doc_id)),
             Self::TextBlock(i) => (i.to_node(doc_id)),
             Self::Code(i) => (i.to_node(doc_id)),
@@ -393,7 +394,12 @@ impl ftd::Scene {
                     if !self.common.is_not_visible {
                         node.style.insert(s("display"), s("block"));
                     }
-                    node.style.insert(s("height"), s("auto"));
+                    if let Some(p) = &self.common.height {
+                        let (key, value) = length(p, "height");
+                        node.style.insert(s(key.as_str()), value);
+                    } else {
+                        node.style.insert(s("height"), s("auto"));
+                    }
                     if let Some(ref data_id) = self.common.data_id {
                         node.attrs
                             .insert(s("data-id"), format!("{}:scene-bg", data_id));
@@ -446,7 +452,7 @@ impl ftd::Scene {
         if self.common.width.is_none() {
             main_node.style.insert(s("width"), s("1000px"));
         }
-        if let Some(ref p) = self.container.spacing {
+        if let Some(ref p) = self.spacing {
             let (key, value) = spacing(p, "margin-left");
             match p {
                 ftd::Spacing::Absolute { value } => {
@@ -463,6 +469,73 @@ impl ftd::Scene {
     }
 }
 
+impl ftd::Grid {
+    pub fn to_node(&self, doc_id: &str) -> Node {
+        let mut n = Node::from_container(&self.common, &self.container, doc_id);
+        if self.inline {
+            n.style.insert(s("display"), s("inline-grid"));
+        } else {
+            n.style.insert(s("display"), s("grid"));
+        }
+        let areas = self
+            .slots
+            .split('|')
+            .map(|v| v.trim())
+            .collect::<Vec<&str>>();
+        let mut css_areas = s("");
+        for area in areas {
+            css_areas = format!("{}'{}'", css_areas, area);
+        }
+        n.style.insert(s("grid-template-areas"), css_areas);
+
+        if let Some(ref columns) = self.slot_widths {
+            n.style
+                .insert(s("grid-template-columns"), s(columns.trim()));
+        }
+        if let Some(ref rows) = self.slot_heights {
+            n.style.insert(s("grid-template-rows"), s(rows.trim()));
+        }
+        if let Some(ref gap) = self.spacing {
+            n.style.insert(s("grid-gap"), format!("{}px", gap));
+        }
+        if let Some(ref gap) = self.spacing_vertical {
+            n.style.insert(s("column-gap"), format!("{}px", gap));
+        }
+        if let Some(ref gap) = self.spacing_horizontal {
+            n.style.insert(s("row-gap"), format!("{}px", gap));
+        }
+        if let Some(ref auto_flow) = self.auto_flow {
+            n.style.insert(s("grid-auto-flow"), s(auto_flow));
+        }
+
+        n.children = {
+            let mut children = vec![];
+            for child in self.container.children.iter() {
+                let mut child_node = child.to_node(doc_id);
+                let common = if let Some(common) = child.get_common() {
+                    common
+                } else {
+                    children.push(child_node);
+                    continue;
+                };
+                if common.anchor.is_some() {
+                    children.push(child_node);
+                    continue;
+                }
+                if let Some(ref position) = common.position {
+                    for (key, value) in grid_align(position) {
+                        child_node.style.insert(s(key.as_str()), value);
+                    }
+                }
+                children.push(child_node);
+            }
+            children
+        };
+
+        n
+    }
+}
+
 impl ftd::Row {
     pub fn to_node(&self, doc_id: &str) -> Node {
         let mut n = Node::from_container(&self.common, &self.container, doc_id);
@@ -480,7 +553,7 @@ impl ftd::Row {
 
         n.style.insert(s("justify-content"), s("flex-start"));
 
-        if let Some(ref p) = self.container.spacing {
+        if let Some(ref p) = self.spacing {
             let (key, value) = spacing(p, "margin-left");
             match p {
                 ftd::Spacing::Absolute { value } => {
@@ -492,6 +565,30 @@ impl ftd::Row {
             };
         }
 
+        n.children = {
+            let mut children = vec![];
+            for child in self.container.children.iter() {
+                let mut child_node = child.to_node(doc_id);
+                let common = if let Some(common) = child.get_common() {
+                    common
+                } else {
+                    children.push(child_node);
+                    continue;
+                };
+                if common.anchor.is_some() {
+                    children.push(child_node);
+                    continue;
+                }
+                if let Some(ref position) = common.position {
+                    for (key, value) in row_align(position) {
+                        child_node.style.insert(s(key.as_str()), value);
+                    }
+                }
+                children.push(child_node);
+            }
+            children
+        };
+
         n
     }
 }
@@ -512,7 +609,7 @@ impl ftd::Column {
 
         n.style.insert(s("justify-content"), s("flex-start"));
 
-        if let Some(ref p) = self.container.spacing {
+        if let Some(ref p) = self.spacing {
             let (key, value) = spacing(p, "margin-top");
             match p {
                 ftd::Spacing::Absolute { value } => {
@@ -524,6 +621,30 @@ impl ftd::Column {
             };
         }
 
+        n.children = {
+            let mut children = vec![];
+            for child in self.container.children.iter() {
+                let mut child_node = child.to_node(doc_id);
+                let common = if let Some(common) = child.get_common() {
+                    common
+                } else {
+                    children.push(child_node);
+                    continue;
+                };
+                if common.anchor.is_some() {
+                    children.push(child_node);
+                    continue;
+                }
+                if let Some(ref position) = common.position {
+                    for (key, value) in column_align(position) {
+                        child_node.style.insert(s(key.as_str()), value);
+                    }
+                }
+                children.push(child_node);
+            }
+            children
+        };
+
         n
     }
 }
@@ -742,9 +863,6 @@ impl ftd::Image {
                 n.style.insert(s("object-position"), s("0 0"));
             }
         }
-        if self.common.width == None {
-            n.style.insert(s("width"), s("100%"));
-        }
 
         n
     }
@@ -935,6 +1053,18 @@ impl ftd::Common {
         if self.link.is_some() {
             d.insert(s("cursor"), s("pointer"));
         }
+        if let Some(p) = &self.z_index {
+            d.insert(s("z-index"), format!("{}px", p));
+        }
+        if let Some(p) = &self.slot {
+            d.insert(s("grid-area"), s(p));
+        }
+        if let Some(p) = &self.grid_column {
+            d.insert(s("grid-column"), s(p));
+        }
+        if let Some(p) = &self.grid_row {
+            d.insert(s("grid-row"), s(p));
+        }
         if self.shadow_size.is_some()
             || self.shadow_blur.is_some()
             || self.shadow_offset_x.is_some()
@@ -982,16 +1112,20 @@ impl ftd::Common {
         }
 
         match &self.anchor {
-            Some(_) => {
-                for (key, value) in non_static_container_align(&self.position, self.inner) {
-                    d.insert(s(key.as_str()), value);
-                }
-            }
-            None => {
-                for (key, value) in container_align(&self.position) {
+            Some(_)
+                if !((self.left.is_some() || self.right.is_some())
+                    && (self.top.is_some() || self.bottom.is_some())) =>
+            {
+                let position = if let Some(ref position) = self.position {
+                    position
+                } else {
+                    &ftd::Position::TopLeft
+                };
+                for (key, value) in non_static_container_align(position, self.inner) {
                     d.insert(s(key.as_str()), value);
                 }
             }
+            _ => {}
         }
 
         let translate = get_translate(
@@ -1191,7 +1325,87 @@ pub fn anchor(l: &ftd::Anchor) -> String {
     }
 }
 
-fn container_align(l: &ftd::Position) -> Vec<(String, String)> {
+fn row_align(l: &ftd::Position) -> Vec<(String, String)> {
+    match l {
+        ftd::Position::Center => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("margin-bottom".to_string(), "auto".to_string()),
+            ("margin-top".to_string(), "auto".to_string()),
+        ],
+        ftd::Position::Top => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("margin-bottom".to_string(), "auto".to_string()),
+        ],
+        ftd::Position::Bottom => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("margin-bottom".to_string(), "0".to_string()),
+            ("margin-top".to_string(), "auto".to_string()),
+        ],
+        _ => vec![],
+    }
+}
+
+pub(crate) fn column_align(l: &ftd::Position) -> Vec<(String, String)> {
+    match l {
+        ftd::Position::Center => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("margin-left".to_string(), "auto".to_string()),
+            ("margin-right".to_string(), "auto".to_string()),
+        ],
+        ftd::Position::Left => vec![
+            ("align-self".to_string(), "flex-start".to_string()),
+            ("margin-right".to_string(), "auto".to_string()),
+        ],
+        ftd::Position::Right => vec![
+            ("align-self".to_string(), "flex-end".to_string()),
+            ("margin-left".to_string(), "auto".to_string()),
+        ],
+        _ => vec![],
+    }
+}
+
+fn grid_align(l: &ftd::Position) -> Vec<(String, String)> {
+    match l {
+        ftd::Position::Center => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("justify-self".to_string(), "center".to_string()),
+        ],
+        ftd::Position::Top => vec![
+            ("align-self".to_string(), "start".to_string()),
+            ("justify-self".to_string(), "center".to_string()),
+        ],
+        ftd::Position::Left => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("justify-self".to_string(), "start".to_string()),
+        ],
+        ftd::Position::Right => vec![
+            ("align-self".to_string(), "center".to_string()),
+            ("justify-self".to_string(), "end".to_string()),
+        ],
+        ftd::Position::Bottom => vec![
+            ("align-self".to_string(), "end".to_string()),
+            ("justify-self".to_string(), "center".to_string()),
+        ],
+        ftd::Position::TopLeft => vec![
+            ("align-self".to_string(), "flex-start".to_string()),
+            ("justify-self".to_string(), "flex-start".to_string()),
+        ],
+        ftd::Position::TopRight => vec![
+            ("align-self".to_string(), "flex-start".to_string()),
+            ("justify-self".to_string(), "flex-end".to_string()),
+        ],
+        ftd::Position::BottomLeft => vec![
+            ("align-self".to_string(), "flex-end".to_string()),
+            ("justify-self".to_string(), "flex-start".to_string()),
+        ],
+        ftd::Position::BottomRight => vec![
+            ("align-self".to_string(), "flex-end".to_string()),
+            ("justify-self".to_string(), "flex-end".to_string()),
+        ],
+    }
+}
+
+/*fn container_align(l: &ftd::Position) -> Vec<(String, String)> {
     match l {
         ftd::Position::Center => vec![
             ("align-self".to_string(), "center".to_string()),
@@ -1235,7 +1449,7 @@ fn container_align(l: &ftd::Position) -> Vec<(String, String)> {
             ("margin-left".to_string(), "auto".to_string()),
         ],
     }
-}
+}*/
 
 fn non_static_container_align(l: &ftd::Position, inner: bool) -> Vec<(String, String)> {
     match l {
@@ -1279,13 +1493,13 @@ fn non_static_container_align(l: &ftd::Position, inner: bool) -> Vec<(String, St
                 vec![
                     ("right".to_string(), "0".to_string()),
                     ("top".to_string(), "50%".to_string()),
-                    ("transform".to_string(), "translate(-50%)".to_string()),
+                    ("transform".to_string(), "translateY(-50%)".to_string()),
                 ]
             } else {
                 vec![
                     ("left".to_string(), "100%".to_string()),
                     ("top".to_string(), "50%".to_string()),
-                    ("transform".to_string(), "translate(-50%)".to_string()),
+                    ("transform".to_string(), "translateY(-50%)".to_string()),
                 ]
             }
         }
@@ -1293,13 +1507,13 @@ fn non_static_container_align(l: &ftd::Position, inner: bool) -> Vec<(String, St
             if inner {
                 vec![
                     ("bottom".to_string(), "0".to_string()),
-                    ("left".to_string(), "50".to_string()),
+                    ("left".to_string(), "50%".to_string()),
                     ("transform".to_string(), "translateX(-50%)".to_string()),
                 ]
             } else {
                 vec![
                     ("top".to_string(), "100%".to_string()),
-                    ("left".to_string(), "50".to_string()),
+                    ("left".to_string(), "50%".to_string()),
                     ("transform".to_string(), "translateX(-50%)".to_string()),
                 ]
             }
