@@ -30,16 +30,18 @@ pub use or_type::OrType;
 pub use rt::RT;
 pub use ui::{
     Anchor, AttributeType, Code, Color, Column, Common, ConditionalAttribute, ConditionalValue,
-    Container, Element, ExternalFont, FontDisplay, GradientDirection, IFrame, Image, Input, Length,
-    NamedFont, Overflow, Position, Region, Row, Scene, Spacing, Style, Text, TextAlign, TextBlock,
-    TextFormat, Weight,
+    Container, Element, ExternalFont, FontDisplay, GradientDirection, Grid, IFrame, Image, Input,
+    Length, NamedFont, Overflow, Position, Region, Row, Scene, Spacing, Style, Text, TextAlign,
+    TextBlock, TextFormat, Weight,
 };
 pub use variable::{PropertyValue, TextSource, Value, Variable};
 
 pub fn js() -> &'static str {
     include_str!("../ftd.js")
 }
-
+pub fn css() -> &'static str {
+    include_str!("../ftd.css")
+}
 pub fn html() -> &'static str {
     include_str!("../ftd.html")
 }
