@@ -88,9 +88,31 @@ impl Document {
     }
 
     pub fn html(&self, id: &str, doc_id: &str) -> String {
-        self.main
-            .to_node(doc_id)
-            .to_html(&Default::default(), &self.rt_data(), id)
+        let mut node = self.main.to_node(doc_id);
+        node.children = {
+            let mut children = vec![];
+            for child in self.main.container.children.iter() {
+                let mut child_node = child.to_node(doc_id);
+                let common = if let Some(common) = child.get_common() {
+                    common
+                } else {
+                    children.push(child_node);
+                    continue;
+                };
+                if common.anchor.is_some() {
+                    children.push(child_node);
+                    continue;
+                }
+                if let Some(ref position) = common.position {
+                    for (key, value) in ftd::html::column_align(position) {
+                        child_node.style.insert(key.as_str().to_string(), value);
+                    }
+                }
+                children.push(child_node);
+            }
+            children
+        };
+        node.to_html(&Default::default(), &self.rt_data(), id)
     }
 
     pub fn set_string(&mut self, name: &str, value: &str) {
@@ -181,6 +203,15 @@ impl Document {
                             return Some(t);
                         }
                     }
+                    ftd::Element::Grid(c) => {
+                        if let Some(v) = f(e) {
+                            return Some(v);
+                        }
+
+                        if let Some(t) = finder(&c.container.children, f) {
+                            return Some(t);
+                        }
+                    }
                     ftd::Element::Image(_) => {
                         if let Some(v) = f(e) {
                             return Some(v);
@@ -311,15 +342,22 @@ impl Document {
 
     pub fn title(&self) -> Option<ftd::Rendered> {
         // find the text of first primary heading
-        if let Some(t) =
-            Self::get_heading(&self.main.container.children, &|r| r.is_primary_heading())
-        {
-            return Some(t);
-        }
-
-        // find any heading
-        if let Some(t) = Self::get_heading(&self.main.container.children, &|r| r.is_heading()) {
-            return Some(t);
+        for i in vec![
+            ftd::Region::H0,
+            ftd::Region::H1,
+            ftd::Region::H2,
+            ftd::Region::H3,
+            ftd::Region::H4,
+            ftd::Region::H5,
+            ftd::Region::H6,
+            ftd::Region::H7,
+        ] {
+            if let Some(t) = Self::get_heading(
+                &self.main.container.children,
+                &|r| matches!(r, r if r == &i),
+            ) {
+                return Some(t);
+            }
         }
 
         // find any text with caption
@@ -563,7 +601,8 @@ pub fn default_scene_children_position(elements: &mut Vec<ftd::Element>) {
         match element {
             ftd::Element::Scene(ftd::Scene { container, .. })
             | ftd::Element::Row(ftd::Row { container, .. })
-            | ftd::Element::Column(ftd::Column { container, .. }) => {
+            | ftd::Element::Column(ftd::Column { container, .. })
+            | ftd::Element::Grid(ftd::Grid { container, .. }) => {
                 default_scene_children_position(&mut container.children);
                 if let Some((_, _, ref mut ext_children)) = container.external_children {
                     default_scene_children_position(ext_children);
