@@ -24,17 +24,35 @@ pub fn common_from_properties(
             .collect(),
         None => vec![],
     };
+
     let anchor = ftd::Anchor::from(
         ftd::p2::utils::string_optional("anchor", properties, doc.name, 0)?,
         doc.name,
     )?;
 
-    let inner_default = match anchor {
-        Some(ref p) => match p {
-            ftd::Anchor::Parent => false,
-            ftd::Anchor::Window => true,
-        },
-        None => false,
+    let (position, inner) = {
+        let mut position = None;
+        let mut inner = match anchor {
+            Some(ref p) => match p {
+                ftd::Anchor::Parent => false,
+                ftd::Anchor::Window => true,
+            },
+            None => false,
+        };
+        let position_inner =
+            match ftd::p2::utils::string_optional("position", properties, doc.name, 0)? {
+                None => ftd::p2::utils::string_optional("align", properties, doc.name, 0)?,
+                Some(v) => Some(v),
+            };
+        if let Some(position_inner) = position_inner {
+            if let Some(p) = position_inner.strip_prefix("inner ") {
+                position = ftd::Position::from(Some(p.to_string()), doc.name)?;
+                inner = true;
+            } else {
+                position = ftd::Position::from(Some(position_inner), doc.name)?;
+            }
+        }
+        (position, inner)
     };
 
     let arguments = {
@@ -244,14 +262,12 @@ pub fn common_from_properties(
         move_down: ftd::p2::utils::int_optional("move-down", properties, doc.name, 0)?,
         move_left: ftd::p2::utils::int_optional("move-left", properties, doc.name, 0)?,
         move_right: ftd::p2::utils::int_optional("move-right", properties, doc.name, 0)?,
-        position: ftd::Position::from(
-            match ftd::p2::utils::string_optional("position", properties, doc.name, 0)? {
-                None => ftd::p2::utils::string_optional("align", properties, doc.name, 0)?,
-                Some(v) => Some(v),
-            },
-            doc.name,
-        )?,
-        inner: ftd::p2::utils::bool_with_default("inner", inner_default, properties, doc.name, 0)?,
+        position,
+        inner,
+        z_index: ftd::p2::utils::int_optional("z-index", properties, doc.name, 0)?,
+        slot: ftd::p2::utils::string_optional("slot", properties, doc.name, 0)?,
+        grid_column: ftd::p2::utils::string_optional("grid-column", properties, doc.name, 0)?,
+        grid_row: ftd::p2::utils::string_optional("grid-row", properties, doc.name, 0)?,
     })
 }
 
@@ -497,9 +513,18 @@ fn common_arguments() -> Vec<(String, ftd::p2::Kind)> {
             ftd::p2::Kind::string().into_optional(),
         ),
         (
-            "inner".to_string(),
-            ftd::p2::Kind::boolean().into_optional(),
+            "z-index".to_string(),
+            ftd::p2::Kind::integer().into_optional(),
         ),
+        ("slot".to_string(), ftd::p2::Kind::string().into_optional()),
+        /*(
+            "grid-column".to_string(),
+            ftd::p2::Kind::string().into_optional(),
+        ),
+        (
+            "grid-row".to_string(),
+            ftd::p2::Kind::string().into_optional(),
+        ),*/
     ]
 }
 
@@ -520,9 +545,6 @@ pub fn container_from_properties(
         children: Default::default(),
         external_children: Default::default(),
         open: ftd::p2::utils::string_bool_optional("open", properties, doc.name, 0)?,
-        spacing: ftd::Spacing::from(ftd::p2::utils::string_optional(
-            "spacing", properties, doc.name, 0,
-        )?)?,
         wrap: ftd::p2::utils::bool_with_default("wrap", false, properties, doc.name, 0)?,
     })
 }
@@ -530,10 +552,6 @@ pub fn container_from_properties(
 fn container_arguments() -> Vec<(String, ftd::p2::Kind)> {
     vec![
         ("open".to_string(), ftd::p2::Kind::string().into_optional()),
-        (
-            "spacing".to_string(),
-            ftd::p2::Kind::string().into_optional(),
-        ),
         ("align".to_string(), ftd::p2::Kind::string().into_optional()),
         ("wrap".to_string(), ftd::p2::Kind::boolean().into_optional()),
     ]
@@ -597,10 +615,17 @@ pub fn row_function() -> ftd::Component {
         kernel: true,
         full_name: "ftd#row".to_string(),
         root: "ftd.kernel".to_string(),
-        arguments: [container_arguments(), common_arguments()]
-            .concat()
-            .into_iter()
-            .collect(),
+        arguments: [
+            container_arguments(),
+            common_arguments(),
+            vec![(
+                "spacing".to_string(),
+                ftd::p2::Kind::string().into_optional(),
+            )],
+        ]
+        .concat()
+        .into_iter()
+        .collect(),
         locals: Default::default(),
         properties: Default::default(),
         instructions: Default::default(),
@@ -626,6 +651,9 @@ pub fn row_from_properties(
             properties, doc, condition, is_child, events, all_locals, root_name, None,
         )?,
         container: container_from_properties(properties, doc)?,
+        spacing: ftd::Spacing::from(ftd::p2::utils::string_optional(
+            "spacing", properties, doc.name, 0,
+        )?)?,
     })
 }
 
@@ -635,10 +663,17 @@ pub fn column_function() -> ftd::Component {
         kernel: true,
         full_name: "ftd#column".to_string(),
         root: "ftd.kernel".to_string(),
-        arguments: [container_arguments(), common_arguments()]
-            .concat()
-            .into_iter()
-            .collect(),
+        arguments: [
+            container_arguments(),
+            common_arguments(),
+            vec![(
+                "spacing".to_string(),
+                ftd::p2::Kind::string().into_optional(),
+            )],
+        ]
+        .concat()
+        .into_iter()
+        .collect(),
         locals: Default::default(),
         properties: Default::default(),
         instructions: Default::default(),
@@ -663,6 +698,9 @@ pub fn column_from_properties(
             properties, doc, condition, is_child, events, all_locals, root_name, None,
         )?,
         container: container_from_properties(properties, doc)?,
+        spacing: ftd::Spacing::from(ftd::p2::utils::string_optional(
+            "spacing", properties, doc.name, 0,
+        )?)?,
     })
 }
 
@@ -1325,11 +1363,17 @@ pub fn decimal_function() -> ftd::Component {
 
 pub fn scene_function() -> ftd::Component {
     let arguments = {
-        let mut arguments: std::collections::BTreeMap<String, ftd::p2::Kind> =
-            [container_arguments(), common_arguments()]
-                .concat()
-                .into_iter()
-                .collect();
+        let mut arguments: std::collections::BTreeMap<String, ftd::p2::Kind> = [
+            container_arguments(),
+            common_arguments(),
+            vec![(
+                "spacing".to_string(),
+                ftd::p2::Kind::string().into_optional(),
+            )],
+        ]
+        .concat()
+        .into_iter()
+        .collect();
         arguments.remove("spacing");
         arguments.remove("wrap");
         arguments
@@ -1350,6 +1394,49 @@ pub fn scene_function() -> ftd::Component {
     }
 }
 
+pub fn grid_function() -> ftd::Component {
+    let arguments: std::collections::BTreeMap<String, ftd::p2::Kind> = [
+        container_arguments(),
+        common_arguments(),
+        vec![
+            ("slots".to_string(), ftd::p2::Kind::string()),
+            (
+                "slot-widths".to_string(),
+                ftd::p2::Kind::string().into_optional(),
+            ),
+            (
+                "slot-heights".to_string(),
+                ftd::p2::Kind::string().into_optional(),
+            ),
+            (
+                "spacing".to_string(),
+                ftd::p2::Kind::integer().into_optional(),
+            ),
+            (
+                "inline".to_string(),
+                ftd::p2::Kind::boolean().into_optional(),
+            ),
+        ],
+    ]
+    .concat()
+    .into_iter()
+    .collect();
+
+    ftd::Component {
+        line_number: 0,
+        kernel: true,
+        root: "ftd.kernel".to_string(),
+        full_name: "ftd#grid".to_string(),
+        arguments,
+        locals: Default::default(),
+        properties: Default::default(),
+        instructions: Default::default(),
+        invocations: Default::default(),
+        condition: None,
+        events: vec![],
+    }
+}
+
 pub fn boolean_function() -> ftd::Component {
     ftd::Component {
         line_number: 0,
@@ -1454,5 +1541,47 @@ pub fn scene_from_properties(
             properties, doc, condition, is_child, events, all_locals, root_name, None,
         )?,
         container: container_from_properties(properties, doc)?,
+        spacing: ftd::Spacing::from(ftd::p2::utils::string_optional(
+            "spacing", properties, doc.name, 0,
+        )?)?,
+    })
+}
+
+pub fn grid_from_properties(
+    properties_with_ref: &std::collections::BTreeMap<String, (ftd::Value, Option<String>)>,
+    doc: &ftd::p2::TDoc,
+    condition: &Option<ftd::p2::Boolean>,
+    is_child: bool,
+    events: &[ftd::p2::Event],
+    all_locals: &mut ftd::Map,
+    root_name: Option<&str>,
+) -> ftd::p1::Result<ftd::Grid> {
+    let properties = &ftd::p2::utils::properties(properties_with_ref);
+    Ok(ftd::Grid {
+        slots: match ftd::p2::utils::string_optional("slots", properties, doc.name, 0)? {
+            Some(val) => val,
+            None => return ftd::e2("expected slots", doc.name, 0),
+        },
+        slot_widths: ftd::p2::utils::string_optional("slot-widths", properties, doc.name, 0)?,
+        slot_heights: ftd::p2::utils::string_optional("slot-heights", properties, doc.name, 0)?,
+        spacing: ftd::p2::utils::int_optional("spacing", properties, doc.name, 0)?,
+        spacing_vertical: ftd::p2::utils::int_optional(
+            "spacing-vertical",
+            properties,
+            doc.name,
+            0,
+        )?,
+        spacing_horizontal: ftd::p2::utils::int_optional(
+            "spacing-horizontal",
+            properties,
+            doc.name,
+            0,
+        )?,
+        common: common_from_properties(
+            properties, doc, condition, is_child, events, all_locals, root_name, None,
+        )?,
+        container: container_from_properties(properties, doc)?,
+        inline: ftd::p2::utils::bool_with_default("inline", false, properties, doc.name, 0)?,
+        auto_flow: ftd::p2::utils::string_optional("auto-flow", properties, doc.name, 0)?,
     })
 }
