@@ -13,6 +13,7 @@ pub enum Element {
     Boolean(Text),
     Decimal(Text),
     Scene(Scene),
+    Grid(Grid),
     Null,
 }
 
@@ -48,15 +49,22 @@ impl Element {
                 Self::Row(ftd::Row {
                     common: ftd::Common { data_id: id, .. },
                     container,
+                    ..
                 })
                 | Self::Column(ftd::Column {
                     common: ftd::Common { data_id: id, .. },
                     container,
+                    ..
                 })
                 | Self::Scene(ftd::Scene {
                     common: ftd::Common { data_id: id, .. },
                     container,
                     ..
+                })
+                | Self::Grid(ftd::Grid {
+                    common: ftd::Common { data_id: id, .. },
+                    container,
+                    ..
                 }) => {
                     let mut index_vec = index_vec.to_vec();
                     index_vec.push(idx);
@@ -142,6 +150,7 @@ impl Element {
                         children,
                         ..
                     },
+                ..
             })
             | Self::Column(ftd::Column {
                 common: ftd::Common { data_id: id, .. },
@@ -151,6 +160,7 @@ impl Element {
                         children,
                         ..
                     },
+                ..
             })
             | Self::Scene(ftd::Scene {
                 common: ftd::Common { data_id: id, .. },
@@ -161,6 +171,16 @@ impl Element {
                         ..
                     },
                 ..
+            })
+            | Self::Grid(ftd::Grid {
+                common: ftd::Common { data_id: id, .. },
+                container:
+                    ftd::Container {
+                        external_children,
+                        children,
+                        ..
+                    },
+                ..
             }) => (
                 id,
                 external_children
@@ -256,6 +276,7 @@ impl Element {
                 ftd::Element::Row(ftd::Row { container, .. }) => container,
                 ftd::Element::Column(ftd::Column { container, .. }) => container,
                 ftd::Element::Scene(ftd::Scene { container, .. }) => container,
+                ftd::Element::Grid(ftd::Grid { container, .. }) => container,
                 _ => continue,
             };
             let all_locals = ftd::Element::get_external_children_dependencies(&container.children);
@@ -300,6 +321,7 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
                 })
                 | ftd::Element::Row(ftd::Row {
                     common:
@@ -309,6 +331,7 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
                 })
                 | ftd::Element::Scene(ftd::Scene {
                     common:
@@ -318,6 +341,17 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
+                })
+                | ftd::Element::Grid(ftd::Grid {
+                    common:
+                        ftd::Common {
+                            conditional_attribute,
+                            data_id: id,
+                            ..
+                        },
+                    container,
+                    ..
                 }) => {
                     ftd::Element::get_style_event_dependencies(&container.children, data);
                     if let Some((_, _, external_children)) = &container.external_children {
@@ -395,6 +429,7 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
                 })
                 | ftd::Element::Row(ftd::Row {
                     common:
@@ -404,6 +439,7 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
                 })
                 | ftd::Element::Scene(ftd::Scene {
                     common:
@@ -413,6 +449,17 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
+                })
+                | ftd::Element::Grid(ftd::Grid {
+                    common:
+                        ftd::Common {
+                            reference,
+                            data_id: id,
+                            ..
+                        },
+                    container,
+                    ..
                 }) => {
                     ftd::Element::get_value_event_dependencies(&container.children, data);
                     if let Some((_, _, external_children)) = &container.external_children {
@@ -469,6 +516,7 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
                 })
                 | ftd::Element::Row(ftd::Row {
                     common:
@@ -478,6 +526,7 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
                 })
                 | ftd::Element::Scene(ftd::Scene {
                     common:
@@ -487,6 +536,17 @@ impl Element {
                             ..
                         },
                     container,
+                    ..
+                })
+                | ftd::Element::Grid(ftd::Grid {
+                    common:
+                        ftd::Common {
+                            condition,
+                            data_id: id,
+                            ..
+                        },
+                    container,
+                    ..
                 }) => {
                     ftd::Element::get_visible_event_dependencies(&container.children, data);
                     if let Some((_, _, external_children)) = &container.external_children {
@@ -538,14 +598,22 @@ impl Element {
                 ftd::Element::Row(ftd::Row {
                     common: ftd::Common { locals, .. },
                     container,
+                    ..
                 })
                 | ftd::Element::Column(ftd::Column {
                     common: ftd::Common { locals, .. },
                     container,
+                    ..
                 })
                 | ftd::Element::Scene(ftd::Scene {
                     common: ftd::Common { locals, .. },
                     container,
+                    ..
+                })
+                | ftd::Element::Grid(ftd::Grid {
+                    common: ftd::Common { locals, .. },
+                    container,
+                    ..
                 }) => {
                     let mut all_locals = ftd::Element::get_locals(&container.children);
                     for (k, v) in locals {
@@ -582,6 +650,7 @@ impl Element {
             ftd::Element::Column(e) => e.container.is_open(is_container_children_empty),
             ftd::Element::Row(e) => e.container.is_open(is_container_children_empty),
             ftd::Element::Scene(e) => e.container.is_open(is_container_children_empty),
+            ftd::Element::Grid(e) => e.container.is_open(is_container_children_empty),
             _ => (false, None),
         }
     }
@@ -591,6 +660,7 @@ impl Element {
             ftd::Element::Column(e) => e.common.data_id.clone(),
             ftd::Element::Row(e) => e.common.data_id.clone(),
             ftd::Element::Scene(e) => e.common.data_id.clone(),
+            ftd::Element::Grid(e) => e.common.data_id.clone(),
             _ => None,
         }
     }
@@ -600,6 +670,7 @@ impl Element {
             ftd::Element::Column(e) => e.common.data_id = name,
             ftd::Element::Row(e) => e.common.data_id = name,
             ftd::Element::Scene(e) => e.common.data_id = name,
+            ftd::Element::Grid(e) => e.common.data_id = name,
             _ => {}
         }
     }
@@ -617,7 +688,8 @@ impl Element {
             | ftd::Element::Integer(ftd::Text { common, .. })
             | ftd::Element::Boolean(ftd::Text { common, .. })
             | ftd::Element::Decimal(ftd::Text { common, .. })
-            | ftd::Element::Scene(ftd::Scene { common, .. }) => common.id = name,
+            | ftd::Element::Scene(ftd::Scene { common, .. })
+            | ftd::Element::Grid(ftd::Grid { common, .. }) => common.id = name,
             ftd::Element::Null => {}
         }
     }
@@ -635,7 +707,8 @@ impl Element {
             | ftd::Element::Integer(ftd::Text { common, .. })
             | ftd::Element::Boolean(ftd::Text { common, .. })
             | ftd::Element::Decimal(ftd::Text { common, .. })
-            | ftd::Element::Scene(ftd::Scene { common, .. }) => common,
+            | ftd::Element::Scene(ftd::Scene { common, .. })
+            | ftd::Element::Grid(ftd::Grid { common, .. }) => common,
             ftd::Element::Null => return,
         }
         .condition = condition;
@@ -654,7 +727,8 @@ impl Element {
             | ftd::Element::Integer(ftd::Text { common, .. })
             | ftd::Element::Boolean(ftd::Text { common, .. })
             | ftd::Element::Decimal(ftd::Text { common, .. })
-            | ftd::Element::Scene(ftd::Scene { common, .. }) => common,
+            | ftd::Element::Scene(ftd::Scene { common, .. })
+            | ftd::Element::Grid(ftd::Grid { common, .. }) => common,
             ftd::Element::Null => return,
         }
         .is_not_visible = is_not_visible;
@@ -673,7 +747,8 @@ impl Element {
             | ftd::Element::Integer(ftd::Text { common, .. })
             | ftd::Element::Boolean(ftd::Text { common, .. })
             | ftd::Element::Decimal(ftd::Text { common, .. })
-            | ftd::Element::Scene(ftd::Scene { common, .. }) => common,
+            | ftd::Element::Scene(ftd::Scene { common, .. })
+            | ftd::Element::Grid(ftd::Grid { common, .. }) => common,
             ftd::Element::Null => return,
         }
         .locals = locals;
@@ -692,7 +767,8 @@ impl Element {
             | ftd::Element::Integer(ftd::Text { common, .. })
             | ftd::Element::Boolean(ftd::Text { common, .. })
             | ftd::Element::Decimal(ftd::Text { common, .. })
-            | ftd::Element::Scene(ftd::Scene { common, .. }) => common,
+            | ftd::Element::Scene(ftd::Scene { common, .. })
+            | ftd::Element::Grid(ftd::Grid { common, .. }) => common,
             ftd::Element::Null => return,
         }
         .events
@@ -721,6 +797,7 @@ impl Element {
             ftd::Element::Boolean(e) => Some(&mut e.common),
             ftd::Element::Decimal(e) => Some(&mut e.common),
             ftd::Element::Scene(e) => Some(&mut e.common),
+            ftd::Element::Grid(e) => Some(&mut e.common),
             _ => None,
         }
     }
@@ -739,6 +816,7 @@ impl Element {
             ftd::Element::Boolean(e) => Some(&e.common),
             ftd::Element::Decimal(e) => Some(&e.common),
             ftd::Element::Scene(e) => Some(&e.common),
+            ftd::Element::Grid(e) => Some(&e.common),
             _ => None,
         }
     }
@@ -908,19 +986,19 @@ impl Default for Position {
 }
 
 impl Position {
-    pub fn from(l: Option<String>, doc_id: &str) -> ftd::p1::Result<ftd::Position> {
+    pub fn from(l: Option<String>, doc_id: &str) -> ftd::p1::Result<Option<ftd::Position>> {
         Ok(match l.as_deref() {
-            Some("center") => Self::Center,
-            Some("top") => Self::Top,
-            Some("bottom") => Self::Bottom,
-            Some("left") => Self::Left,
-            Some("right") => Self::Right,
-            Some("top-left") => Self::TopLeft,
-            Some("top-right") => Self::TopRight,
-            Some("bottom-left") => Self::BottomLeft,
-            Some("bottom-right") => Self::BottomRight,
+            Some("center") => Some(Self::Center),
+            Some("top") => Some(Self::Top),
+            Some("bottom") => Some(Self::Bottom),
+            Some("left") => Some(Self::Left),
+            Some("right") => Some(Self::Right),
+            Some("top-left") => Some(Self::TopLeft),
+            Some("top-right") => Some(Self::TopRight),
+            Some("bottom-left") => Some(Self::BottomLeft),
+            Some("bottom-right") => Some(Self::BottomRight),
             Some(t) => return ftd::e2(format!("{} is not a valid alignment", t), doc_id, 0), // TODO
-            None => return Ok(Self::TopLeft),
+            None => None,
         })
     }
 }
@@ -1263,8 +1341,12 @@ pub struct Common {
     pub move_down: Option<i64>,
     pub move_left: Option<i64>,
     pub move_right: Option<i64>,
-    pub position: Position,
+    pub position: Option<Position>,
     pub inner: bool,
+    pub z_index: Option<i64>,
+    pub slot: Option<String>,
+    pub grid_column: Option<String>,
+    pub grid_row: Option<String>,
     // TODO: background-image, un-cropped, tiled, tiled{X, Y}
     // TODO: border-style: solid, dashed, dotted
     // TODO: border-{shadow, glow}
@@ -1298,7 +1380,6 @@ pub struct Container {
     pub children: Vec<ftd::Element>,
     pub external_children: Option<(String, Vec<Vec<usize>>, Vec<ftd::Element>)>,
     pub open: (Option<bool>, Option<String>),
-    pub spacing: Option<Spacing>,
     pub wrap: bool,
 }
 
@@ -1324,11 +1405,27 @@ pub struct Image {
 #[derive(serde::Deserialize, Debug, Default, PartialEq, Clone, serde::Serialize)]
 pub struct Row {
     pub container: Container,
+    pub spacing: Option<Spacing>,
     pub common: Common,
 }
 
 #[derive(serde::Deserialize, Debug, Default, PartialEq, Clone, serde::Serialize)]
 pub struct Scene {
+    pub container: Container,
+    pub spacing: Option<Spacing>,
+    pub common: Common,
+}
+
+#[derive(serde::Deserialize, Debug, Default, PartialEq, Clone, serde::Serialize)]
+pub struct Grid {
+    pub slots: String,
+    pub slot_widths: Option<String>,
+    pub slot_heights: Option<String>,
+    pub spacing: Option<i64>,
+    pub spacing_vertical: Option<i64>,
+    pub spacing_horizontal: Option<i64>,
+    pub inline: bool,
+    pub auto_flow: Option<String>,
     pub container: Container,
     pub common: Common,
 }
@@ -1336,6 +1433,7 @@ pub struct Scene {
 #[derive(serde::Deserialize, Debug, PartialEq, Clone, Default, serde::Serialize)]
 pub struct Column {
     pub container: Container,
+    pub spacing: Option<Spacing>,
     pub common: Common,
 }
 
